TestCase subclass: #ObjTest
	instanceVariableNames: 'objectClass aPoint coloredPointClass pointClass classClass '
	classVariableNames: ''
	poolDictionaries: 'ObjClassEnvironment '
	category: 'ObjVLispApp'!


!ObjTest methodsFor: 'setup'!

assembleClassClass

	classClass := ObjExtension new: 6.
	classClass at: classClass offsetForName put: #ObjClass.
	classClass at: classClass offsetForClass put: #ObjClass.
	classClass at: classClass offsetForIVs
		put: #(#class #name #superclass #iv #keywords #methodDict).
	classClass at: classClass offsetForKeywords
		put: #(#name: #superclass: #iv: #keywords: #methodDict:).
	classClass at: classClass offsetForSuperclass put: #ObjObject.
	classClass at: classClass offsetForMethodDict
		put: (IdentityDictionary new: 3).
	ObjExtension declareClass: classClass!

assembleColoredPointClass
	
	coloredPointClass := ObjExtension new: 6.
	coloredPointClass at: pointClass offsetForName put: #ObjColoredPoint.
	coloredPointClass at: pointClass offsetForClass put: #ObjClass.
	coloredPointClass at: pointClass offsetForIVs put: #(#color).
	coloredPointClass at: pointClass offsetForSuperclass put: #ObjPoint.
	coloredPointClass 
		at: pointClass offsetForMethodDict
		put: (IdentityDictionary new: 3).
	ObjExtension declareClass: coloredPointClass.

	(coloredPointClass at: (coloredPointClass offsetForMethodDict)) at: #print
		put: 
			[:objself | 
			Transcript show: 'I''am a colored point';
				cr]!

assembleObjectClass

	objectClass := ObjExtension new: 6.
	objectClass at: objectClass offsetForName put: #ObjObject.
	objectClass at: objectClass offsetForClass put: #ObjClass.
	objectClass at: objectClass offsetForIVs put: #(#class).
	objectClass at: objectClass offsetForKeywords put: #().
	objectClass at: objectClass offsetForSuperclass put: #ObjObject.
	objectClass at: objectClass offsetForMethodDict
		put: (IdentityDictionary new: 3).
	ObjExtension declareClass: objectClass.
	(objectClass at: objectClass offsetForMethodDict) at: #print
		put: 
			[:objself | 
			Transcript show: 'I''am an Object';
				cr].
	(objectClass at: objectClass offsetForMethodDict) at: #error
		put: 
		[:object :selector | Transcript show: 'Error: selector ', selector asString, 'not understood' ; cr].
	objectClass addMethod: #getIV
		withBody: [:object :iv | object valueOfInstanceVariable: iv].

	objectClass addMethod: #setIV
		withBody: 
			[:object :iv :val | 
			object at: (object offsetFromObjectOfInstanceVariable: iv) put: val].!

assemblePointClass

	pointClass := ObjExtension new: 6.
	pointClass at: pointClass offsetForName put: #ObjPoint.
	pointClass at: pointClass offsetForClass put: #ObjClass.
	pointClass at: pointClass offsetForIVs put: #(#class #x #y).
	pointClass at: pointClass offsetForKeywords put: #(#x: #y:).
	pointClass at: pointClass offsetForSuperclass put: #ObjObject.
	pointClass at: pointClass offsetForMethodDict
		put: (IdentityDictionary new: 3).
	ObjExtension declareClass: pointClass.

	(pointClass at: pointClass offsetForMethodDict) at: #x
		put: [:objself | objself valueOfInstanceVariable: #x].
	(pointClass at: pointClass offsetForMethodDict) at: #print
		put: 
			[:objself | 
			Transcript show: 'I''am a Point';
				cr]!

assemblePointInstance


	aPoint := ObjExtension new: 3.
	aPoint at: 1 put: #ObjPoint .
	aPoint at:2 put: 10.
	aPoint at:3 put: 15!

setUp
	"self new setUp"

	ObjExtension initialize.
	self assembleClassClass.
	self assemblePointClass.
	self assembleObjectClass.
	self assembleColoredPointClass.
	self assemblePointInstance! !

!ObjTest methodsFor: 'tests'!

testAllocate
	"(self  selector: #testAllocate) run"

	| newInstance testInstance |
	testInstance := (ObjExtension new: 3).
	testInstance  at: 1 put:  #ObjPoint.
	newInstance := pointClass allocateAnInstance.
	self should: [newInstance =  testInstance].
	self should: [(newInstance objClass) = pointClass]!

testClassAccess
	"(self  selector: #testClassAccess) run"
	
	self should: [aPoint objClass = pointClass]!

testInitialize
	"(self  selector: #testInitialize) run"

	| newInstance testInstance |
	newInstance := pointClass allocateAnInstance.
	testInstance := ObjExtension new: 3.
	testInstance at: 1 put: #ObjPoint.
	testInstance at: 2 put: 1.
	testInstance at: 3 put: 2.
	newInstance initializeUsing:  #(#y: 2 #z: 3 #t: 55 #x: 1).
	self should: [newInstance = testInstance]!

testInstanceVariableInheritance
	"(self  selector: #testInstanceVariableInheritance) run"

	self should: 
			[(ObjExtension new computeNewIVFrom: #(#a #b #c #d) asOrderedCollection
				with: #(#a #z #b #t) asOrderedCollection)
					= #(#a #b #c #d #z #t) asOrderedCollection].
	self should: 
			[(ObjExtension new computeNewIVFrom: #() asOrderedCollection
				with: #(#a #z #b #t) asOrderedCollection)
					= #(#a #z #b #t) asOrderedCollection]!

testIVOffset
	"(self  selector: #testIVOffset) run"

	self should: [(pointClass offsetFromClassOfInstanceVariable: #x ) = 2].
	self should: [(pointClass offsetFromClassOfInstanceVariable: #lulu ) = 0]!

testIVOffsetAndValue
	"(self  selector: #testIVOffsetAndValue) run"

	self should: [(aPoint offsetFromObjectOfInstanceVariable: #x ) = 2].
	self should: [(aPoint valueOfInstanceVariable: #x ) = 10]!

testKeywords
	"(self  selector: #testKeywords) run"

	|dummyObject|
	dummyObject := ObjExtension new.
	self should: [(dummyObject generateKeywords: #(titi toto lulu)) = #(#titi: #toto: #lulu:)].
	self should:  [(dummyObject
					 keywordValue: #x getFrom: #(#toto 33 #x 23)  ifAbsent: 2) = 23].
	self should: [(dummyObject  
					keywordValue: #x getFrom: #(#toto 23) ifAbsent: 2) = 2].
	self should:  [(dummyObject 
					returnValuesFrom: #(#x 22 #y 35)
					followingSchema: #(#y #yy #x #y)) = #(35 nil 22 35)]!

testMethodLookup
	"(self  selector: #testMethodLookup) run"


	self should:  [ pointClass lookup: #x for: aPoint.
				pointClass methodFoundIn = pointClass ].

	self should:  [ coloredPointClass lookup: #print for: aPoint.
				coloredPointClass classToLookForSuperSend = pointClass].
	self should: 
			[coloredPointClass lookup: #x for: aPoint.
			coloredPointClass methodFoundIn = pointClass]	
	"we cannot test the error for the moment because ObjObject does exist
	at that time"!

testMethodManagment
	"(self  selector: #testMethodManagment) run"

	self should: [pointClass doesUnderstand: #x].
	self shouldnt: [pointClass doesUnderstand: #xx].
	pointClass addMethod: #xx
		withBody: [:objself | objself valueOfInstanceVariable: #x ].

	self should: [((pointClass bodyOfMethod: #xx) value: aPoint) = 10].
	self should: [pointClass doesUnderstand: #xx].
	pointClass removeMethod: #xx.
	self shouldnt: [pointClass doesUnderstand: #xx].
	self should: [((pointClass bodyOfMethod: #x) value: aPoint) = 10]!

testMethodSelfSend
	"(self  selector: #testMethodSelfSend) run"

	self should: [(aPoint unarySend: #x) = 10].
	self should: [(aPoint binarySend: #getIV with: #x) = 10].
	self should: [ (aPoint send: #setIV withArguments: #(y 22)).
				(aPoint binarySend: #getIV with: #y) = 22]!

testPrimitiveStructure
	"(self selector: #testPrimitiveStructure) run"

	self should: [pointClass objClassId = #ObjClass].
	self should: [(pointClass objName ) = #ObjPoint].
	self should: [(pointClass objSuperclassId ) = #ObjObject].
	self should: [(pointClass objIVs) = #(#class #x #y)].
	self should: [(pointClass objKeywords) = #(#x: #y:)].
	self shouldnt: [(pointClass objMethodDict) = nil].! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObjTest class
	instanceVariableNames: ''!


!ObjTest class methodsFor: 'run'!

allTestCases
	"self allTestCases"
	| result |
	result := OrderedCollection new.
	result add: (self selector: #testAllocate).
	result add: (self selector: #testClassAccess).
	result add: (self selector: #testIVOffset).
	result add: (self selector: #testIVOffsetAndValue).
	result add: (self selector: #testKeywords).
	result add: (self selector: #testPrimitiveStructure).
	"result add: (self selector: #testMethodLookup)."
	result add: (self selector: #testMethodManagment).
	result add: (self selector: #testMethodSelfSend).
	^result!

runAll
	"self runAll"

	| test |
	test := TestSuite named: 'Complete ObL Test Suite'.
	test addTestCases: self allTestCases.
	^test run! !

Inspector subclass: #ObjClassInspector
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'ObjClassEnvironment '
	category: 'ObjVLispApp'!
ObjClassInspector comment:
'ObjClassInspector is special inspector to help inspecting ObjClass'!


!ObjClassInspector methodsFor: 'private'!

listOfClassFields

	 "object objClass objIVs"
	|class|
	^ object size isZero
		ifTrue: [#()]
		ifFalse: [ (object at: 1) isNil
			ifTrue: [#()]
			ifFalse: [class := (ObjExtension giveClassNamed: (object at: 1) ifAbsent: [nil]).
					class isNil
						ifTrue: [#()]
						ifFalse: [|iv|
								iv := class at: 4.
								iv isNil
									ifTrue: [ #(class name superclass iv keywords methods)]
									ifFalse: [ class at: 4] ]]]! !

!ObjClassInspector methodsFor: 'field list'!

fieldIndex
	"Answer the offset corresponding to the currently selected field."

	^ self listOfClassFields indexOf: field!

fieldList
	"Answer an Array consisting of 'self' and the instance variable 
	names of the inspected object. Up to 40 indices are given for 
	variable length objects."

	^ ((OrderedCollection with: 'self') addAll: self listOfClassFields ;yourself) asArray!

fieldValue

	(field = 'self' or: [field == nil]) ifTrue: [^object].
	^ object basicAt: self fieldIndex! !

Array variableSubclass: #ObjExtension
	instanceVariableNames: ''
	classVariableNames: 'MethodFoundIn '
	poolDictionaries: 'ObjClassEnvironment '
	category: 'ObjVLispApp'!


!ObjExtension methodsFor: 'debugging'!

debug
	"(self giveClassNamed: #ObjClass) debug"

	"(self giveClassNamed: #ObjSet) debug"

	"ObjExtension new debug"

	"|aPt|
	aPt := ObjExtension new: 3.
	aPt at: 1 put: #ObjPoint.
	aPt debug"

	"|aPt|
	aPt := ObjExtension new: 3.
	aPt at: 1 put: #ObjPoint3.
	aPt debug"

	ObjClassInspector openOn: self! !

!ObjExtension methodsFor: 'class creation'!

name: aSymbol superclass: aClassName iv: anOrderedCollection 
	"(ObjExtension giveClassNamed: #ObjClass) name: #ObjPoint2 superclass: #ObjObject iv: #(#x #y)"

	| array |
	array := #(#name: nil #iv: nil #superclass: nil).
	array at: 2 put: aSymbol.
	array at: 4 put: anOrderedCollection.
	array at: 6 put: aClassName.
	self send: #new withArguments:  (Array with: array)! !

!ObjExtension methodsFor: 'instance allocation'!

allocateAnInstance
	"Returns a newly created instance of self, an ObjClass. In this implementation the identifier of the object class is the name of the class. "

	| a |
	a := ObjExtension new: (self numberOfIVs).
	a objClassId:  (self objName).
	^a! !

!ObjExtension methodsFor: 'printing'!

printOn: aStream

	aStream nextPutAll: 'an Obj object: '.	
	super printOn: aStream! !

!ObjExtension methodsFor: 'class structure offset'!

offsetForIVs
	^4!

offsetForKeywords

	^5!

offsetForMethodDict

	^6!

offsetForName

	^2!

offsetForSuperclass

	^3! !

!ObjExtension methodsFor: 'message passing'!

binarySend: selector with: argument 
	"send the message whose selector is <selector> to the receiver. The arguments of the messages
	are an  <argument>. The method is lookep up in the class of the receiver. 
	self is an objObject or a objClass."

	^(self objClass lookup: selector for: self) value: self value: argument!

binarySuper: selector with: argument 
	"send the message whose selector is <selector> to the receiver. The arguments of the messages
	are an  <argument>. The method is lookep up in the class of the receiver. 
	self is an objObject or a objClass."

	^(self classToLookForSuperSend lookup: selector for: self) value: self
		value: argument!

send: selector withArguments: arguments 
	"send the message whose selector is <selector> to the receiver. The arguments of the messages
	are an array <arguments>. The method is lookep up in the class of the receiver. 
	self is an objObject or a objClass."

	^(self objClass lookup: selector for: self)
				valueWithArguments: (Array with: self) , arguments!

super: selector withArguments: arguments 
	"Invoke an oveeriden method named <selector> with an array of arguments <arguments>. 
	self is an objClass."

	^(self classToLookForSuperSend lookup: selector for: self)
				valueWithArguments: (Array with: self) ,arguments!

unarySend: selector 
	^(self objClass lookup: selector for: self) value: self!

unarySuper: selector 
	^(self classToLookForSuperSend lookup: selector for: self) value: self! !

!ObjExtension methodsFor: 'object structure offset'!

offsetForClass

	^1! !

!ObjExtension methodsFor: 'object structure primitive'!

objClass
	"Returns the ObjClass object of the object and not its internal identification. Differs from classId  which is a primitive 
	to access the structure representing a class"

	^ObjExtension giveClassNamed: self objClassId!

objClassId
	"Returns the identifier that identifies the class of the object. In this implementation we chose to use the class name as identifier"

	^self at: self offsetForClass!

objClassId: anObjClassId
	"Set the identifier that identifies the class of the object."
	
	self at: self offsetForClass put: anObjClassId! !

!ObjExtension methodsFor: 'method management'!

addMethod: aSelector withBody: aBlock 
	"Define a method with selector <aSelector> and a body <aBlock> in the receiver.
	If a method with the same selector already existed, the new definition will erase it.
	self is an objClass"

	self  objMethodDict at: aSelector put: aBlock.
	^self!

bodyOfMethod: aSelector 
	"Return the method associated with the selector <aSelector> in the receiver.
	Return nil if the selector is not defined.  self is an objClass"

	^self objMethodDict at: aSelector ifAbsent: [nil]!

doesUnderstand: aSelector 
	"Tell if the receiver has a method having <aSelector> as selector. self is anObjClass"

	^self objMethodDict keys includes: aSelector!

removeMethod: aSelector
	"Remove the method with aSelector <aSymbol> in the receiver. 
	self is an objClass"

	self objMethodDict removeKey: aSelector ifAbsent: [].
	^self! !

!ObjExtension methodsFor: 'keyword management'!

generateKeywords: anArray 
	"Returns an array containing the keywords made from the collection of Symbol passed as argument"
	"self new generateKeywords: #(titi toto lulu) "

	^anArray collect: [:e | (e , ':') asSymbol]!

keywordValue: aSymbol getFrom: anArray ifAbsent: aDefaultValue 
	"precondition: ((length anArray) mod 2) = 0 
	returns the value associated with the keyword represented by aSymbol 
	in the initarg list represented by anArray."

	"self new keywordValue: #titi getFrom: #(toto 12 titi 23) ifAbsent: 2"
	"self new keywordValue: #titi getFrom: #(toto 23) ifAbsent: 2"

	| i |
	i := anArray indexOf: aSymbol ifAbsent: nil.
	^i isNil 
		ifTrue: [aDefaultValue] 
		ifFalse: [anArray at: i + 1]!

returnValuesFrom: anInitargArray followingSchema: anArrayOfKeywords 
	"Return the values associated with the keys. The extracted values are taken from <anInitargArray> 
	and the return values are extracted according to the schema defined by the collection of keys <anArrayOfKeywords>"

	"self new returnValuesFrom: #(lulu 22  titi 35)  followingSchema: #(titi toto lulu titi)  #(35 nil 22 35)"

	^anArrayOfKeywords collect: 
			[:e | 
			self keywordValue: e
				getFrom: anInitargArray 
				ifAbsent: nil]! !

!ObjExtension methodsFor: 'basic class operations'!

computeAndSetKeywords
	"Compute the keywords of a given class.  As the class of an object should not be changed.
	 the keywords should note contain the keyword class. self is anObjClass"

	self objKeywords: (self generateKeywords: (self objIVs copyWithout: #class))! !

!ObjExtension methodsFor: 'iv management'!

numberOfIVs
	"Returns the number of instance variables of the class an ObjClass"

	^self objIVs size!

offsetFromClassOfInstanceVariable: aSymbol 
	"Returns the index of the instance variable named aSymbol for an class anObjClass.
	Returns 0 if the aSymbol is not present in the instance variable lists of anObjClass"

	^ self objIVs indexOf: aSymbol!

offsetFromObjectOfInstanceVariable: aSymbol
	"Returns the offset of the instance variable named aSymbol in the object anObjObject.
	If aSymbol is not an instance variable is not an instance variable of the object raise an error"

	| aClass |
	aClass := self objClass.
	(aClass objIVs includes: aSymbol) 
		ifFalse: [self error: 'The class ' , aClass objName asString
						, ' does not define the instance variable ' , aSymbol asString].
	^ aClass offsetFromClassOfInstanceVariable: aSymbol!

valueOfInstanceVariable: aSymbol 

	^self at: (self offsetFromObjectOfInstanceVariable: aSymbol)! !

!ObjExtension methodsFor: 'class structure primitive'!

objIVs
	^self at: self offsetForIVs!

objIVs: anOrderedCollection 
	"Set the list of instance variable names of anObjClass"

	self at: self offsetForIVs put: anOrderedCollection!

objKeywords
	"Returns the keyword list of an ObjClass"

	^self at: self offsetForKeywords!

objKeywords: anOrderedCollection
	"Sets the list of keywords of an ObjClass. Note that this method is just an accessor and does not compute the actual list of keywords"

	self at: self offsetForKeywords put: anOrderedCollection!

objMethodDict
	"Returns the method dictionary of an ObjClass"

	^self at: self offsetForMethodDict!

objMethodDict: aDictionary 
	"Sets the method dictionary of an ObjClass"

	self at: self offsetForMethodDict put: aDictionary!

objName
	"Returns the name of an ObjClass"

	^self  at: self offsetForName!

objName: aName 
	"Set the name of an ObjClass"

	self at: self offsetForName put: aName!

objSuperclassId
	"Returns the superclass id of anObjClass"

	^self at: self offsetForSuperclass!

objSuperclassId: anObjClassId
	"Set the superclass id of anObjClass"

	self at: self offsetForSuperclass put: anObjClassId! !

!ObjExtension methodsFor: 'instance initialization'!

initializeUsing: anAlternatedArray 
	"Returns the receiver an ObjObject initialized according to the directives given by anAlternateArray"

	| ivValues |
	ivValues := self  returnValuesFrom: anAlternatedArray
				followingSchema: self objClass objKeywords.
	^ivValues startingAt: 1
		replaceElementsIn: self
		from: 2
		to: ivValues size + 1! !

!ObjExtension methodsFor: 'iv inheritance'!

computeNewIVFrom: superIVOrdCol with: localIVOrdCol
	"Returns an ordered collection that is the union without duplicate of ivOfSuper and ivOrderedCollection.
	Important the order of the instance variable is conserved"

	| ivs |
	^superIVOrdCol isNil 
		ifTrue: [localIVOrdCol]
		ifFalse: 
			[ivs := superIVOrdCol asOrderedCollection copy.
			localIVOrdCol do: [:e | (ivs includes: e) ifFalse: [ivs add: e]].
			ivs]!

computeNewIVFromClass: anObjClass with: ivOrderedCollection 
	"ObL pComputeNewIVFromClass: #(#C #C #O #(a b c d))
									with:#(a z b t) asOrderedCollection"

	"ObL pComputeNewIVFromClass: nil
									with:#(a z b t) asOrderedCollection"

	| ivs |
	^anObjClass isNil
		ifTrue: [ivOrderedCollection]
		ifFalse: 
			[ivs := anObjClass objIVs asOrderedCollection copy.
			ivOrderedCollection do: [:e | (ivs includes: e) ifFalse: [ivs add: e]].
			ivs]! !

!ObjExtension methodsFor: 'method lookup'!

classToLookForSuperSend

	^self class giveClassNamed: MethodFoundIn objSuperclassId!

lookup: selector for: anObjObject 
	"look for the method named <selector> starting in the receiver. 
	The lookup is done for a message sent to <anObjObject>. self is an objClass"

	^(self doesUnderstand: selector)
		ifTrue: 
			[MethodFoundIn := self.	"we mark the class for the super"
			self bodyOfMethod: selector]
		ifFalse: 
			[self objName = #ObjObject
				ifFalse: 
					[(ObjExtension giveClassNamed: self objSuperclassId) 
						lookup: selector
						for: anObjObject]
				ifTrue: 
					[anObjObject 
						binarySend: #error
						with: selector]]! !

!ObjExtension methodsFor: 'for tests'!

methodFoundIn

	^MethodFoundIn! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObjExtension class
	instanceVariableNames: 'methodFoundIn '!


!ObjExtension class methodsFor: 'global class repository management'!

declareClass: anObjClass 
	"To declare an ObjClass in the class repository"

	| nameC |
	nameC := anObjClass objName.
	nameC isNil
		ifFalse: [ObjClassEnvironment at: nameC put: anObjClass]
		ifTrue: [self error: 'The class does not have a name']!

giveClassNamed: aSymbol 
	"Return the class defined in the class repository with the name aSymbol"

	^ self giveClassNamed: aSymbol ifAbsent: [self error: ('The class ' , aSymbol , ' is not defined')]!

giveClassNamed: aSymbol ifAbsent: aBlock

	^ ObjClassEnvironment at: aSymbol ifAbsent: aBlock! !

!ObjExtension class methodsFor: 'bootstrap objObject'!

createObjObject
	| objObject |
	objObject := self objObjectStructure.
	objObject addMethod: #class withBody: [:object | object objClass].
	objObject addMethod: #isClass withBody: [:object | false].
	objObject addMethod: #isMetaclass withBody: [:object | false].
	objObject addMethod: #error
		withBody: 
			[:object :selector | 
			Transcript show: 'Error: selector ' , selector asString , 'not understood'].
	objObject addMethod: #getIV
		withBody: [:object :iv | object valueOfInstanceVariable: iv].
	objObject addMethod: #setIV
		withBody: 
			[:object :iv :val | 
			object at: (object offsetFromObjectOfInstanceVariable: iv) put: val].
	objObject addMethod: #initialize
		withBody: [:object :initargs | object initializeUsing: initargs].
	^objObject!

objObjectStructure
	
	^  ObjClass 
			send: #new
			withArguments: #(#(#name: #ObjObject #iv: #(#class))).! !

!ObjExtension class methodsFor: 'bootstrap objClass'!

createObjClass
	"self bootstrap"

	| objClass |
	objClass := self objClassStructure.
	self defineAllocateMethodIn: objClass.
	self defineNewMethodIn: objClass.
	self defineInitializeMethodIn: objClass.
	objClass addMethod: #isMetaclass
		withBody: [:class | class objIVs includes: #superclass].	"an object is a class if is class is a metaclass. cool"
	objClass addMethod: #isClass
		withBody: [:class | class objClass unarySend: #isMetaclass].
	^objClass!

defineInitializeMethodIn: objClass 
	objClass addMethod: #initialize
		withBody: 
			[:aclass :initArray | 
			aclass binarySuper: #initialize with: initArray.
			aclass objIVs: (aclass
						computeNewIVFrom: (ObjExtension giveClassNamed: aclass objSuperclassId) objIVs
						with: aclass objIVs).
			aclass computeAndSetKeywords.
			aclass objMethodDict: IdentityDictionary new.
			ObjExtension declareClass: aclass.
			aclass]!

objClassStructure

	^ObjClass send: #new
		withArguments: #(#(#name: #ObjClass #iv: #(#name #superclass #iv #keywords #methodDict) #superclass: #ObjObject))! !

!ObjExtension class methodsFor: 'obsolete'!

manuallyCreateObjClass1
	"self manuallyCreateObjClass"

	| class |
	class := ObjExtension new: 6.
	class objClassId: #ObjClass.
	class objName: #ObjClass.
	class objIVs: #(#class #name #superclass #iv #keywords #methodDict).
	class objKeywords: #(#name: #superclass: #iv: #keywords: #methodDict:).
	class objSuperclassId: #ObjObject.
	class objMethodDict: (IdentityDictionary new: 3).
	ObjExtension declareClass: class.
	class addMethod: #initialize
		withBody: 
			[:aclass :initArray | 
			aclass initializeUsing: initArray.	"Initialize a class as an object. In the bootstrapped system will be done via super"
			aclass objIVs: (Obj
						computeNewIVFromClass: (ObjExtension giveClassNamed: aclass objSuperclassId ifAbsent: [nil])
						with: aclass objIVs).
			aclass
				objKeywords: (aclass generateKeywords: (aclass objIVs copyWithout: #class)).
			aclass objMethodDict: (IdentityDictionary new: 3).
			ObjExtension declareClass: aclass.
			aclass].
	class addMethod: #new
		withBody: 
			[:aclass :initArray | 
			aclass allocateAnInstance send: #initialize
				withArguments: (Array with: initArray)].
	class addMethod: #allocate withBody: [:aclass | aclass allocateAnInstance].
	^class!

manuallyCreateObjClass2
	"self manuallyCreateObjClass"

	| class |
	class := ObjExtension new: 6.
	class objClassId: #ObjClass.
	class objName: #ObjClass.
	class objIVs: #(#class #name #superclass #iv #keywords #methodDict).
	class objKeywords: #(#name: #superclass: #iv: #keywords: #methodDict:).
	class objSuperclassId: #ObjObject.
	class objMethodDict: (IdentityDictionary new: 3).
	ObjExtension declareClass: class.
	class addMethod: #initialize
		withBody: 
			[:aclass :initArray | 
			aclass initializeUsing: initArray.	"Initialize a class as an object. In the bootstrapped system will be done via super"
			aclass objMethodDict: (IdentityDictionary new: 3).
			ObjExtension declareClass: aclass.
			aclass].
	class addMethod: #new
		withBody: 
			[:aclass :initArray | 
			aclass allocateAnInstance send: #initialize withArguments: initArray].
	class addMethod: #allocate withBody: [:aclass | aclass allocateAnInstance].
	^class! !

!ObjExtension class methodsFor: 'bootstrap objClass manually'!

bootstrap
	"self bootstrap"
	
	"self initialize."
	self manuallyCreateObjClass.
	self createObjObject.
	self createObjClass.!

defineAllocateMethodIn: class 
	class addMethod: #allocate withBody: [:aclass | aclass allocateAnInstance]!

defineManualInitializeMethodIn: class 
	class addMethod: #initialize
		withBody: 
			[:aclass :initArray | 
			| objsuperclass |
			aclass initializeUsing: initArray.	
			"Initialize a class as an object. In the bootstrapped system will be done via super"
			objsuperclass := ObjExtension giveClassNamed: aclass objSuperclassId ifAbsent: [nil].
			objsuperclass isNil
				ifFalse: 
					[aclass
						objIVs: (aclass computeNewIVFrom: objsuperclass objIVs with: aclass objIVs)]
				ifTrue: 
					[aclass objIVs: (aclass computeNewIVFrom: #(#class) with: aclass objIVs)].
			aclass
				objKeywords: (aclass generateKeywords: (aclass objIVs copyWithout: #class)).
			aclass objMethodDict: (IdentityDictionary new: 3).
			ObjExtension declareClass: aclass.
			aclass]!

defineNewMethodIn: class 
	class addMethod: #new
		withBody: [:aclass :initArray | (aclass unarySend: #allocate) binarySend: #initialize with: initArray]!

manuallyCreateObjClass
	"self manuallyCreateObjClass"

	| class |
	class := self manualObjClassStructure.
	ObjExtension declareClass: class.
	self defineManualInitializeMethodIn: class.	
	self defineNewMethodIn: class.
	self defineAllocateMethodIn: class.
	^class!

manualObjClassStructure
	| class |
	class := ObjExtension new: 6.
	class objClassId: #ObjClass.
	class objName: #ObjClass.
	class objIVs: #(#class #name #superclass #iv #keywords #methodDict).
	class objKeywords: #(#name: #superclass: #iv: #keywords: #methodDict:).
	class objSuperclassId: #ObjObject.
	class objMethodDict: (IdentityDictionary new: 3).
	^class! !

!ObjExtension class methodsFor: 'initialize'!

initialize
	"self initialize"	

	ObjClassEnvironment 
		keysDo: [:each| ObjClassEnvironment removeKey: each]. 
	ObjClassEnvironment at: #ObjClass put: nil.
	ObjClassEnvironment at: #ObjObject put: nil.
	MethodFoundIn := nil.! !

!ObjExtension class methodsFor: 'first user-defined classes'!

newcoloredPointDefinition
	"self newcoloredPointDefinition"

	| coloredPointClass aColoredPoint |
	self newpointDefinition.
	coloredPointClass := ObjClass send: #new
				withArguments: #((#name: #ObjColoredPoint #iv: #(#color) #superclass: #ObjPoint)).

	aColoredPoint := coloredPointClass send: #new withArguments: #((#x: 24 #y: 6 #color: #blue)).	
	"first messages sent"

	aColoredPoint binarySend: #getIV with: #x.
	aColoredPoint send: #setIV withArguments: #(#x 25).
	aColoredPoint binarySend: #getIV with: #x.

	aColoredPoint binarySend: #getIV with: #color.	"adding some methods"

	coloredPointClass addMethod: #giveColor
		withBody: [:objself | objself binarySend: #getIV with: #color].

	coloredPointClass addMethod: #setColor
		withBody: [:objself :val | objself send: #setIV withArguments: (Array with: #color with: val)].

	coloredPointClass addMethod: #display
		withBody: 
			[:objself | 
			objself unarySuper: #display.
			Transcript cr;
				show: ' with Color = '.
			Transcript show: (objself unarySend: #giveColor) printString;
				cr].
	aColoredPoint unarySend: #givex.
	aColoredPoint unarySend: #giveColor.
	aColoredPoint unarySend: #display!

newpointDefinition
	"self newpointDefinition"

	| pointClass aPoint |
	ObjExtension bootstrap.
	pointClass := ObjClass send: #new
				withArguments: #((#name: #ObjPoint #iv: #(#x #y) #superclass: #ObjObject)).

	aPoint := pointClass send: #new withArguments: #((#x: 24 #y: 6)).

	aPoint binarySend: #getIV with: #x.

	aPoint send: #setIV withArguments: #(#x 25).

	aPoint binarySend: #getIV with: #x.

	pointClass addMethod: #givex
		withBody: [:objself | objself binarySend: #getIV with: #x].

	pointClass addMethod: #setx
		withBody: [:objself :val | objself send: #setIV withArguments: (Array with: #x with: val)].

	pointClass addMethod: #display
		withBody: 
			[:objself | 
			Transcript cr;
				show: 'aPoint with x = '.
			Transcript show: (objself unarySend: #givex) printString;
				cr].
	aPoint unarySend: #givex.
	aPoint binarySend: #setx with: #(33).
	aPoint unarySend: #display.
	^aPoint! !

!ObjExtension class methodsFor: 'first user-defined metaclasses'!

abstractMetaclassDefinition
	"self abstractMetaclassDefinition"

	| abstractClass |
	abstractClass := (ObjExtension giveClassNamed: #ObjClass) 
						send: #new
						withArguments: #(#(#name: #ObjAbstractClass #iv: #() #superclass: #ObjClass)).
	abstractClass 
		addMethod: #new
		withBody:  [:class :initArray | 
					class error: ' the class ' , class objName asString , ' is abstract'].!

abstractMetaclassExample
	"self abstractMetaclassExample"

	| abstractPointClass |
	self abstractMetaclassDefinition.
	self newpointDefinition.
	abstractPointClass := (Obj giveClassNamed: #ObjAbstractClass) 
							send: #new
							withArguments: #(#(#name: #ObjAbstractPoint #iv: #() #superclass: #ObjPoint)).
	abstractPointClass 
		send: #new
		withArguments: #(#(#x: 24 #y: 6))	"should raise an error"!

setMetaclassDefinition
	"self setMetaclassDefinition"

	| setClass  |
	setClass :=  (ObjExtension giveClassNamed: #ObjClass)
				send: #new
				withArguments: #(#(#name: #ObjSet #iv: #(#myInstances) #superclass: #ObjClass)).	"initialize on a metaclass"

	setClass  addMethod: #initialize
		withBody: 
			[:class :initArray | 
			class binarySuper: #initialize
				with: initArray.
			class send: #setIV
				  withArguments: (Array with: #myInstances with: OrderedCollection new).
			class].

	setClass addMethod: #instances
		withBody: 
			[:class | 
			class binarySend: #getIV with: #myInstances].

	setClass addMethod: #new
		withBody: 
			[:class :initArray | 
			| newInst others |
			newInst := class  super: #new withArguments:  (Array with: initArray).
			others :=  class unarySend: #instances.
			others := others add: newInst ; yourself.
			class send:  #setIV withArguments: (Array with: #myInstances with: others).
			newInst].!

setMetaclassExample
	"self setMetaclassExample"

	| memoPointClass |
	self setMetaclassDefinition.
	memoPointClass := (Obj giveClassNamed: #ObjSet) send: #new
				withArguments: #(#(#name: #ObjMemoPoint #iv: #() #superclass: #ObjPoint)).
	memoPointClass send: #new withArguments: #(#(#x: 24 #y: 6)).
	memoPointClass send: #new withArguments: #(#(#x: 15 #y: 10)).
	memoPointClass debug.
	^memoPointClass unarySend: #instances! !

Object subclass: #ObjDebug
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjVLispApp'!


!ObjDebug methodsFor: 'ObjClassEnvironment'!

objClassEnvironment
	"self new objClassEnvironment"
	

	(Smalltalk at: #ObjClassEnvironment) inspect! !

!ObjDebug methodsFor: 'emergency'!

fixProblemWithThePoolDictionaryForClass: aSmalltalkClass
	
	"The problem is that the system look for the name of the 
	PoolDictionary in the global environment by looking at the 
	key associated with the pool dictionary object. 
	If this object would changed (if we reassign the 
	value associated with the pool dictionary name)
	the system could not found a key to the old object.Then 
	it would break"

	aSmalltalkClass setSharedPools: Set new.
	aSmalltalkClass addSharedPool: ObjClassEnvironment.! !
"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

ObjDebug class
	instanceVariableNames: ''!


!ObjDebug class methodsFor: 'emergency'!

fixProblemWithPools
	"self fixProblemWithPools"

	self fixProblemWithThePoolDictionaryForClass: Obj.
	self fixProblemWithThePoolDictionaryForClass: ObjExtension.
	self fixProblemWithThePoolDictionaryForClass: ObjClassInspector.
	self fixProblemWithThePoolDictionaryForClass: ObjTest.
	self fixProblemWithThePoolDictionaryForClass: ObjExamples!

fixProblemWithThePoolDictionaryForClass: aSmalltalkClass
	
	"The problem is that the system look for the name of the 
	PoolDictionary in the global environment by looking at the 
	key associated with the pool dictionary object. 
	If this object would changed (if we reassign the 
	value associated with the pool dictionary name)
	the system could not found a key to the old object.Then 
	it would break"

	aSmalltalkClass setSharedPools: Set new.
	aSmalltalkClass addSharedPool: ObjClassEnvironment.! !

TestCase subclass: #ObjTestBootstrap
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'ObjClassEnvironment '
	category: 'ObjVLispApp'!


!ObjTestBootstrap methodsFor: 'test creation ObjObject'!

testCreateObjObjectInstanceMessage
	"(self selector: #testCreateObjObjectInstanceMessage) run"

	| pointClass objClass pointInstance |
	ObjExtension initialize.
	ObjExtension manuallyCreateObjClass.
	ObjExtension createObjObject.
	objClass := ObjExtension giveClassNamed: #ObjClass.
	pointClass := objClass send: #new
				withArguments: #((#name: #ObjPoint #superclass: #ObjObject #iv: #(#x #y))).
	pointInstance := pointClass send: #new withArguments: #(()).
	self should: [pointInstance objClassId = #ObjPoint].
	self should: [(pointInstance  binarySend: #getIV with: #x) = nil].
	self should: [pointInstance send: #setIV withArguments: #(x 25).
				(pointInstance binarySend: #getIV with: #x) = 25]!

testCreateObjObjectMessage
	"(self selector: #testCreateObjObjectMessage) run"

	| objObject |
	ObjExtension initialize.
	ObjExtension manuallyCreateObjClass.
	ObjExtension createObjObject.
	objObject := ObjExtension giveClassNamed: #ObjObject.
	self should: 
			[(objObject unarySend: #class) = (ObjExtension giveClassNamed: #ObjClass)].
	self should: [(objObject unarySend: #isClass) not].
	self should: [(objObject binarySend: #getIV with: #class) = #ObjClass]!

testCreateObjObjectStructure
	"(self selector: #testCreateObjObjectStructure) run"

	| objObject  |
	ObjExtension initialize.
	ObjExtension manuallyCreateObjClass.
	ObjExtension createObjObject. 
	objObject := ObjExtension giveClassNamed: #ObjObject.
	self should: [objObject objName = #ObjObject].
	self should: [objObject objClassId = #ObjClass].
	self should: [objObject objSuperclassId isNil].
	self should: [objObject objIVs asArray = #(class)].
	self should: [objObject objKeywords asArray = #()]! !

!ObjTestBootstrap methodsFor: 'test ObjClass manual creation '!

testManuallyCreateObjClassAllocate
	"(self selector: #testManuallyCreateObjClassAllocate) run"

	| objClass emptyClass |
	ObjExtension initialize.
	ObjExtension manuallyCreateObjClass.
	objClass := ObjExtension giveClassNamed: #ObjClass.
	emptyClass := objClass unarySend: #allocate.
	self should: [emptyClass objClassId = #ObjClass].
	self should: [emptyClass objSuperclassId isNil].
	self should: [emptyClass objIVs isNil].
	self should: [emptyClass objKeywords isNil].
	self should: [emptyClass objMethodDict isNil].
	self should: [emptyClass objName isNil]!

testManuallyCreateObjClassStructure
	"(self selector: #testManuallyCreateObjClassStructure) run"

	| objClass |
	ObjExtension initialize.
	ObjExtension manuallyCreateObjClass.
	objClass := ObjExtension giveClassNamed: #ObjClass.
	self should: [objClass objName = #ObjClass].
	self should: [ objClass objClassId = #ObjClass].
	self should: [objClass objClass == objClass].
	self should: [objClass objSuperclassId = #ObjObject].
	"the fact that the created first class inherits form ObjObject is not necessary
	because there is no super calls"
	self should: [objClass objIVs = #(class #name #superclass # iv #keywords #methodDict)].
	self should:[objClass objKeywords = #(name: superclass: iv: keywords: methodDict:)].! !

!ObjTestBootstrap methodsFor: 'test ObjClass creation'!

testCreateObjClassMessage
	"(self selector: #testCreateObjClassMessage) run"

	| objClass |
	ObjExtension bootstrap.
	objClass := ObjExtension giveClassNamed: #ObjClass.

	self should:  [(objClass unarySend: #class) = (ObjExtension giveClassNamed: #ObjClass)].
	self should: [(objClass unarySend: #isClass)].
	self should: [(objClass unarySend: #isMetaclass)].!

testCreateObjClassStructure
	"(self selector: #testCreateObjClassStructure) run"

	| objClass  |
	ObjExtension bootstrap.
	objClass := ObjExtension giveClassNamed: #ObjClass.
	self should: [objClass objName = #ObjClass].
	self should: [objClass objClassId = #ObjClass].
	self should: [objClass objSuperclassId = #ObjObject].
	self should: [objClass objIVs asArray = #(#class #name #superclass #iv #keywords #methodDict )].
	self should: [objClass objKeywords asArray = #(#name: #superclass: #iv: #keywords: #methodDict:)]! !
ObjExtension initialize!


